Module Main

    ' The database
    Public Database As Database

    ' The name of the current player
    Public PlayerName As String

    ' The current level we're at
    Public Level As Int32

    ' Whether or not we are said to exit the application
    Private ExitApp As Boolean

    ' Our start-up function
    Public Sub Main()

        ' Firstly, initialize all the databases
        Database = New Database(Environment.CurrentDirectory & "\Data\Database.xml")

        ' Let's introduce ourself for the player
        Console.WriteLine("Welcome to Millionaire!")

        ' Initialize the player
        DoAction(2) ' Edit the name

        ' Start the menu loop!
        MenuLoop()

    End Sub

    Private Sub DoAction(ByVal ActionID As Int32)

        ' Do something based upon the given ID
        Select Case ActionID
            Case 1 ' Start a game
                GameLoop()
            Case 2 ' Edit the name
                PlayerName = InquireUser(" - Please state your name: ")
            Case 3 ' Exit
                ExitApp = True
        End Select

    End Sub

    Public Function FindByLevel(ByVal nLevel As Int32) As Collection

        Dim tmpColl As New Collection, oQuestion As Question

        ' Go through all questions and add all corresponding to this level
        For Each oQuestion In Database.Questions
            If oQuestion.Level = nLevel Then
                tmpColl.Add(oQuestion)
            End If
        Next

        ' Return the given collection
        Return tmpColl

    End Function

    Public Sub PrintQuestion(ByVal oQuestion As Question)

        Dim Tell As Int32

        ' Firstly, print the question itself
        Console.WriteLine() ' Some space
        Console.WriteLine("Question " & Level + 1 & ", " & oQuestion.Title _
         & ". Current money: " & Database.MoneyList(Level + 1) & Database.MoneyCurrency)
        Console.WriteLine(" - " & oQuestion.Description)

        ' Then print all the answers
        For Tell = 0 To oQuestion.AnswerCount
            Console.WriteLine("   " & oQuestion.AnswersID(Tell) & ". " & oQuestion.AnswersText(Tell))
        Next

        ' Inform the user that a tool may be used
        Console.WriteLine("   (Press 1 to use a tool, 2 to give up.)")

    End Sub

    Public Sub ToolDialog(ByVal oQuestion As Question)

        Dim sTool As String, lngID As Int32

        ' Let the user choose a tool to use
        Console.WriteLine(" - What tool do you want to use?")

        ' Add all tools
        For Each sTool In Database.Tools.ToolDescription

            ' Increase the tool-ID
            lngID = lngID + 1

            ' Make sure this tool is available
            If Not Database.Tools.ToolUnavailable(lngID - 1) Then
                Console.WriteLine("   " & lngID & ". " & sTool)
            End If

        Next

        ' Let the user choose what to use
        Console.WriteLine(Database.Tools.ExecuteTool(Convert.ToInt32(Val(InquireUser("Choose one: "))) - 1, oQuestion))

    End Sub

    Public Sub GameLoop()

        Dim currentQuestion As Question, currLevel As Collection
        Dim oRandom As New Random, sAnswer As String, bEnd As Boolean

        ' Clear variables
        Level = 0

        ' Run till we've reached the end
        Do Until (Level > Database.MoneyList.Count)

            ' ToDo: Write the current state

            ' Enumate all questions at this level
            currLevel = FindByLevel(Level)

            ' If there's no questions at this level, assume the user has won
            If currLevel.Count <= 0 Then
                Level = Database.MoneyList.Count - 1
                GameEnd()
                Exit Do
            End If

            ' Use a random question from this level
            currentQuestion = currLevel(Convert.ToInt32((oRandom.NextDouble * (currLevel.Count - 1)) + 1))

            ' Print this question
            PrintQuestion(currentQuestion)

            ' Make sure the user is confident in his/her answer
            Do
                ' Get an answer
                sAnswer = GetAnswer(bEnd, currentQuestion)

                ' If we are to quit the current game, quit it
                If bEnd Then
                    Exit Sub
                End If

                ' Ask the user
                If InquireUser("Is that your final answer (Y, N)? ").ToUpper = "Y" Then
                    Exit Do
                End If

            Loop

            ' See if the given answer is correct
            If sAnswer = currentQuestion.AnswersSolution Then
                Console.WriteLine("Correct! You now have " & Database.MoneyList(Level + 2) & Database.MoneyCurrency)
            Else
                GameEnd("Incorrect, I'm afraid! The answer was " & currentQuestion.AnswersSolution & ".", False)
                Exit Do
            End If

            ' Go to the next level
            Level = Level + 1

        Loop

    End Sub

    Public Function GetAnswer(ByRef bEnd As Boolean, ByVal currentQuestion As Question) As String

        Dim sAnswer As String

        Do
            ' Wait for an answer
            sAnswer = InquireUser("What's your answer: ")

            ' See what to do
            Select Case sAnswer
                Case "1" ' Use a tool
                    ToolDialog(currentQuestion)
                Case "2" ' Give up
                    GameEnd("A wise choice. The answer was " & currentQuestion.AnswersSolution & ".", True)
                    bEnd = True
                    Return ""
                Case Else ' Return the answer
                    Return sAnswer
            End Select

        Loop

    End Function

    ' Inform the user that she/he has won/lost
    Public Sub GameEnd(Optional ByVal Text As String = "Congratulations! You won!", Optional ByVal bLost As Boolean = False)

        Dim iMoney As Int32

        ' Save the current amount of money
        iMoney = Database.MoneyList(Level + 1)

        ' If the user lost, ...
        If bLost Then

            ' ... we'll have to find the closest guarenteed money
            iMoney = Database.MoneyGuaranteed(FindClosest(Database.MoneyGuaranteed, iMoney))

        End If

        Console.WriteLine(Text & " You got " & _
         iMoney & Database.MoneyCurrency & ".")
        Console.WriteLine(Environment.NewLine)

    End Sub

    Public Function FindClosest(ByVal refColl As Collection, ByVal Find As Int32) As Int32

        Dim Tell As Int32

        ' Find the nearest value
        For Tell = 1 To refColl.Count

            ' If the current value is higher than what we're looking for, ...
            If refColl(Tell) > Find Then

                ' ... obviously, the last value is lower
                Return Tell - 1

            End If

        Next

        ' We'll just assume the closest value is the last
        Return refColl.Count

    End Function

    Public Sub MenuLoop()

        ' The main game loop
        Do Until ExitApp

            ' Clear variable(s)
            Database.Tools.NewGame()

            ' Print the available actions
            Console.WriteLine("What would you like to do, " & PlayerName & "?")
            Console.WriteLine(" - 1: Start a new game")
            Console.WriteLine(" - 2: Edit your name")
            Console.WriteLine(" - 3: Exit")

            ' Ask what the user would like to do
            DoAction(Convert.ToInt32(Val(InquireUser("Choose one: ")))) ' Yeah, yeah. I know, but it works!

        Loop

    End Sub

    Public Function InquireUser(ByVal sQuestion As String) As String

        ' Firstly, print the question
        Console.Write(sQuestion)

        ' Then wait for user input
        Return Console.ReadLine()

    End Function

End Module
